﻿Imports Microsoft.Maps.MapControl.WPF
Imports System.Net.Http
Imports Microsoft.Kinect
Imports System.ComponentModel

Class MainWindow
    Private myKinect As KinectSensor
    Private myBodyFrameReader As BodyFrameReader
    Private myBodies As Body()

    Private myHandPositionX As Double
    Private myHandPositionY As Double
    Private myColorSpacePoint As ColorSpacePoint
    Private myBytesPerPixel As Integer = 4 'フルカラーの場合は32ビットで1バイトは8ビットなので4バイトとなる

    Private myColorFrameReader As ColorFrameReader = Nothing
    Private colorBitmap As WriteableBitmap = Nothing
    Private ColorImagePixelData As Byte()
    Private BytesPerPixel As Integer = 4

    Private commonJoint As Joint
    Private point As DepthSpacePoint
    Private Index As Integer = 0
    Private no As Integer = 0
    Private Async Sub MainWindow_Loaded(sender As Object, e As RoutedEventArgs) Handles Me.Loaded
        Dim client As New HttpClient
        Dim myPin As New Pushpin
        Dim myAddress As String = "愛媛県松山市道後"
        Dim addressUri = String.Format("http://www.geocoding.jp/api/?v=1.1&q={0}", myAddress)
        Dim myResponse = Await client.GetStringAsync(New Uri(addressUri, UriKind.Absolute))
        Dim doc = XElement.Parse(myResponse)
        Dim myAddressLatitude = doc.Descendants("coordinate").Elements("lat").Value
        Dim myAddressLongitude = doc.Descendants("coordinate").Elements("lng").Value

        Dim myLabel As New Label
        With myLabel
            .Content = myAddress
            .Foreground = New SolidColorBrush(Colors.Navy)
            .Background = New SolidColorBrush(Colors.Beige)
            .FontSize = 20

        End With

        Dim myLocation As New Location
        myLocation.Latitude = myAddressLatitude '緯度
        myLocation.Longitude = myAddressLongitude '経度
        myPin.Location = New Location(myAddressLatitude, myAddressLongitude)
        MapLayer.SetPosition(myLabel, myLocation)

        myMap.SetView(myLocation, 16)
        myMap.Children.Add(myPin)
        myMap.Children.Add(myLabel)

        myKinect = KinectSensor.GetDefault
        If myKinect Is Nothing = False Then
            Dim myDepthDescription = myKinect.DepthFrameSource.FrameDescription
            myKinect.Open()
            myBodyFrameReader = myKinect.BodyFrameSource.OpenReader
            AddHandler myBodyFrameReader.FrameArrived, AddressOf myBodyFrameReader_FrameArrived
            myBodies = New Body(myKinect.BodyFrameSource.BodyCount - 1) {}
            myColorFrameReader = myKinect.ColorFrameSource.OpenReader
            Dim myColorFrameDescripton As FrameDescription = myKinect.ColorFrameSource.CreateFrameDescription(ColorImageFormat.Bgra)
            ColorImagePixelData = New Byte(myColorFrameDescripton.Width * myColorFrameDescripton.Height * BytesPerPixel - 1) {}
            colorBitmap = New WriteableBitmap(myColorFrameDescripton.Width, myColorFrameDescripton.Height, 96.0, 96.0, PixelFormats.Bgr32, Nothing)

        End If
    End Sub

    Private Sub myBodyFrameReader_FrameArrived(sender As Object, e As BodyFrameArrivedEventArgs)
        'CanvasBody.Children.Clear()
        Using myBodyFrame = e.FrameReference.AcquireFrame
            If myBodyFrame Is Nothing = True Then
                Return
            End If
            myBodyFrame.GetAndRefreshBodyData(myBodies)
        End Using
        CanvasBody.Children.Clear()
        For Each body In myBodies

            For Each joint In body.Joints
                If joint.Value.TrackingState = TrackingState.Tracked Then
                    DrawEllipse(joint.Value, 10, Brushes.Transparent)
                    If joint.Value.JointType = JointType.HandRight Then
                        If body.Joints(JointType.HandTipRight).TrackingState = TrackingState.Tracked Then
                            myColorSpacePoint = myKinect.CoordinateMapper.MapCameraPointToColorSpace(body.Joints(JointType.HandTipRight).Position)
                            myHandPositionX = CInt(myColorSpacePoint.X * 0.5) 'CInt((body.Joints(JointType.ThumbRight).Position.X))
                            myHandPositionY = CInt(myColorSpacePoint.Y * 0.5) 'CInt((body.Joints(JointType.ThumbRight).Position.Y))
                            DrawHandState(body.Joints(JointType.HandRight), body.HandRightState, body.HandRightConfidence)

                        End If

                    End If
                End If
            Next
        Next
    End Sub

    Private Sub DrawHandState(joint As Joint, state As HandState, myTrackingConfidence As TrackingConfidence)
        If myTrackingConfidence <> TrackingConfidence.High Then
            Return
        End If

        If state = HandState.Open Then
            Index = 1
        ElseIf state = HandState.Closed Then
            Index = 2
        Else
            Index = 3
        End If
    End Sub
    Private Sub DrawEllipse(myJoint As Joint, R As Integer, myBrush As Brush)

        Dim myEllipse = New Ellipse
        With myEllipse
            .Width = R
            .Height = R
            .Fill = myBrush
        End With

        point = myKinect.CoordinateMapper.MapCameraPointToDepthSpace(myJoint.Position)
        If point.X < 0 OrElse point.Y < 0 Then
            Return
        End If
        Canvas.SetLeft(myEllipse, point.X - (R / 2))
        Canvas.SetTop(myEllipse, point.Y - (R / 2))
        If myJoint.JointType = JointType.HandRight Then

            Select Case Index
                Case 1
                    myMap.ZoomLevel += 0.1
                Case 2
                    myMap.ZoomLevel -= 0.1
                Case 3
                    Index = 0
                    Exit Sub
            End Select
        End If
        CanvasBody.Children.Add(myEllipse)
    End Sub
    Private Sub MainWindow_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        If myKinect Is Nothing = False Then
            myKinect.Close()
            myKinect = Nothing
        End If
    End Sub
End Class
